@@ -11,29 +11,37 @@
  */
 
 
+#ifdef USE_QT5_INDI
+#include <baseclientqt.h>
+#else
 #include <baseclient.h>
+#endif
 
 #include "ui_ekosmanager.h"
+
+#include "ekos.h"
 #include "indi/indistd.h"
-#include "capture.h"
-#include "focus.h"
-#include "guide.h"
-#include "align.h"
-#include "mount.h"
-#include "dome.h"
-#include "weather.h"
-#include "dustcap.h"
-#include "scheduler.h"
+#include "capture/capture.h"
+#include "focus/focus.h"
+#include "guide/guide.h"
+#include "align/align.h"
+#include "mount/mount.h"
+#include "auxiliary/dome.h"
+#include "auxiliary/weather.h"
+#include "auxiliary/dustcap.h"
+#include "scheduler/scheduler.h"
 
 #include <QDialog>
 #include <QHash>
 #include <QtDBus/QtDBus>
 
 class DriverInfo;
+class ProfileInfo;
 class KPageWidgetItem;
+class QProgressIndicator;
 
 /**
- *@class EkosManager
+ *@class Manager
  *@short Primary class to handle all Ekos modules.
  * The Ekos Manager class manages startup and shutdown of INDI devices and registeration of devices within Ekos Modules. Ekos module consist of \ref Ekos::Mount, \ref Ekos::Capture, \ref Ekos::Focus, \ref Ekos::Guide, and \ref Ekos::Align modules.
  * \ref EkosDBusInterface "Ekos DBus Interface" provides high level functions to control devices and Ekos modules for a total robotic operation:
@@ -47,8 +55,9 @@ class KPageWidgetItem;
  * <li>\ref DustCapDBusInterface "Dust Cap DBus Interface"</li>
  * </ul>
  *  For low level access to INDI devices, the \ref INDIDBusInterface "INDI Dbus Interface" provides complete access to INDI devices and properties.
+ *  Ekos Manager provides a summary of operations progress in the <i>Summary</i> section of the <i>Setup</i> tab.
  *@author Jasem Mutlaq
- *@version 1.2
+ *@version 1.5
  */
 class EkosManager : public QDialog, public Ui::EkosManager
 {
@@ -56,13 +65,13 @@ class EkosManager : public QDialog, public Ui::EkosManager
     Q_CLASSINFO("D-Bus Interface", "org.kde.kstars.Ekos")
 
 public:
-    EkosManager();
+    EkosManager(QWidget *parent);
     ~EkosManager();
 
-    typedef enum { STATUS_IDLE, STATUS_PENDING, STATUS_SUCCESS, STATUS_ERROR } CommunicationStatus;
+    typedef enum { EKOS_STATUS_IDLE, EKOS_STATUS_PENDING, EKOS_STATUS_SUCCESS, EKOS_STATUS_ERROR } CommunicationStatus;
 
     void appendLogText(const QString &);
-    void refreshRemoteDrivers();
+    //void refreshRemoteDrivers();
     void setOptionsWidget(KPageWidgetItem *ops) { ekosOption = ops; }
     void addObjectToScheduler(SkyObject *object);
 
@@ -79,11 +88,17 @@ public:
     /*@{*/
 
     /** DBUS interface function.
-     * set Ekos connection mode.
-     * @param isLocal if true, it will establish INDI server locally, otherwise it will connect to a remote INDI server as defined in the Ekos options or by the user.
-     * /note This function must be called before all functions in Ekos DBUS Interface.
+     * set Current device profile.
+     * @param profileName Profile name
+     * @return True if profile is set, false if not found.
      */
-    Q_SCRIPTABLE Q_NOREPLY void setConnectionMode(bool isLocal);
+    Q_SCRIPTABLE bool setProfile(const QString &profileName);
+
+    /** DBUS interface function
+     * @brief getProfiles Return a list of all device profiles
+     * @return List of device profiles
+     */
+    Q_SCRIPTABLE QStringList getProfiles();
 
     /** DBUS interface function.
      * @retrun INDI connection status (0 Idle, 1 Pending, 2 Connected, 3 Error)
@@ -107,66 +122,11 @@ public:
      */
     Q_SCRIPTABLE bool stop();
 
-    /** DBUS interface function.
-     * Sets the telescope driver name. If connection mode is local, it is selected from the local drivers combo box. Otherwise, it is set as the remote telescope driver.
-     * @param telescopeName telescope driver name. For remote devices, the name has to be exactly as the name defined by the driver on startup.
-     */
-    Q_SCRIPTABLE Q_NOREPLY void setTelescope(const QString & telescopeName);
-
-    /** DBUS interface function.
-     * Sets the CCD driver name. If connection mode is local, it is selected from the local drivers combo box. Otherwise, it is set as the remote CCD driver.
-     * @param CCDName CCD driver name. For remote devices, the name has to be exactly as the name defined by the driver on startup.
-     */
-    Q_SCRIPTABLE Q_NOREPLY void setCCD(const QString & ccdName);
-
-    /** DBUS interface function.
-     * Sets the guider driver name. If connection mode is local, it is selected from the local drivers combo box. Otherwise, it is set as the remote guider driver.
-     * @param guiderName guider CCD driver name. For remote devices, the name has to be exactly as the name defined by the driver on startup. If the primary CCD has a guide chip,
-     * do not set the guider name as the guide chip will be automatically selected as guider
-     */
-    Q_SCRIPTABLE Q_NOREPLY void setGuider(const QString & guiderName);
-
-    /** DBUS interface function.
-     * Sets the focuser driver name. If connection mode is local, it is selected from the local drivers combo box. Otherwise, it is set as the remote focuser driver.
-     * @param focuserName focuser driver name. For remote devices, the name has to be exactly as the name defined by the driver on startup.
-     */
-    Q_SCRIPTABLE Q_NOREPLY void setFocuser(const QString & focuserName);
-
-    /** DBUS interface function.
-     * Sets the AO driver name. If connection mode is local, it is selected from the local drivers combo box. Otherwise, it is set as the remote AO driver.
-     * @param AOName Adaptive Optics driver name. For remote devices, the name has to be exactly as the name defined by the driver on startup.
-     */
-    Q_SCRIPTABLE Q_NOREPLY void setAO(const QString & AOName);
-
-    /** DBUS interface function.
-     * Sets the filter driver name. If connection mode is local, it is selected from the local drivers combo box. Otherwise, it is set as the remote filter driver.
-     * @param filterName filter driver name. For remote devices, the name has to be exactly as the name defined by the driver on startup.
-     */
-    Q_SCRIPTABLE Q_NOREPLY void setFilter(const QString & filterName);
-
-    /** DBUS interface function.
-     * Sets the dome driver name. If connection mode is local, it is selected from the local drivers combo box. Otherwise, it is set as the remote dome driver.
-     * @param domeName dome driver name. For remote devices, the name has to be exactly as the name defined by the driver on startup.
-     */
-    Q_SCRIPTABLE Q_NOREPLY void setDome(const QString & domeName);
-
-    /** DBUS interface function.
-     * Sets the weather driver name. If connection mode is local, it is selected from the local drivers combo box. Otherwise, it is set as the remote dome driver.
-     * @param domeName dome driver name. For remote devices, the name has to be exactly as the name defined by the driver on startup.
-     */
-    Q_SCRIPTABLE Q_NOREPLY void setWeather(const QString & domeName);
-
-    /** DBUS interface function.
-     * Sets the auxiliary driver name. If connection mode is local, it is selected from the local drivers combo box. Otherwise, it is set as the remote auxiliary driver.
-     * @param index 1 for Aux 1, 2 for Aux 2, 3 for Aux 3
-     * @param auxiliaryName auxiliary driver name. For remote devices, the name has to be exactly as the name defined by the driver on startup.
-     */
-    Q_SCRIPTABLE Q_NOREPLY void setAuxiliary(int index, const QString & auxiliaryName);    
-
 protected:
     void closeEvent(QCloseEvent *);
     void hideEvent(QHideEvent *);
-    void showEvent(QShowEvent *);    
+    void showEvent(QShowEvent *);
+    void resizeEvent(QResizeEvent *);
 
 public slots:
 
@@ -182,33 +142,33 @@ public slots:
 
     /** @}*/
 
-    void processINDI();   
-    void cleanDevices();
+    void processINDI();
+    void cleanDevices(bool stopDrivers=true);
 
     void processNewDevice(ISD::GDInterface*);
     void processNewProperty(INDI::Property*);
     void processNewNumber(INumberVectorProperty *nvp);
     void processNewText(ITextVectorProperty *tvp);
 
-protected slots:
+private slots:
+
+    void changeAlwaysOnTop(Qt::ApplicationState state);
 
     void updateLog();
     void clearLog();
 
     void processTabChange();
 
+    void processServerTermination(const QString &host, const QString &port);
+
     void removeDevice(ISD::GDInterface*);
 
     void deviceConnected();
     void deviceDisconnected();
 
-    void processINDIModeChange();
+    //void processINDIModeChange();
     void checkINDITimeout();
 
-    void toggleINDIPanel();
-    void toggleFITSViewer();
-    void checkFITSViewerState();
-
     void setTelescope(ISD::GDInterface *);
     void setCCD(ISD::GDInterface *);
     void setFilter(ISD::GDInterface *);
@@ -217,7 +177,34 @@ protected slots:
     void setWeather(ISD::GDInterface *);
     void setDustCap(ISD::GDInterface *);
     void setLightBox(ISD::GDInterface *);
-    void setST4(ISD::ST4 *);    
+    void setST4(ISD::ST4 *);
+
+    // Profiles
+    void addProfile();
+    void editProfile();
+    void deleteProfile();
+    void saveDefaultProfile(const QString& name);
+
+    // Mount Summary
+    void updateMountCoords(const QString &ra, const QString &dec ,const QString &az ,const QString &alt);
+    void updateMountStatus(ISD::Telescope::TelescopeStatus status);
+    void setTarget(SkyObject *o);
+
+    // Capture Summary
+    void updateCaptureStatus(Ekos::CaptureState status);
+    void updateCaptureProgress(QImage *image, Ekos::SequenceJob *job);
+    void updateExposureProgress(Ekos::SequenceJob *job);
+    void updateCaptureCountDown();
+
+    // Focus summary
+    void setFocusStatus(Ekos::FocusState status);
+    void updateFocusStarPixmap(QPixmap &starPixmap);
+    void updateFocusProfilePixmap(QPixmap &profilePixmap);
+
+    // Guide Summary
+    void updateGuideStatus(Ekos::GuideState status);
+    void updateGuideStarPixmap(QPixmap &starPix);
+    void updateGuideProfilePixmap(QPixmap &profilePix);
 
  private:
 
@@ -232,11 +219,8 @@ protected slots:
     void initWeather();
     void initDustCap();
 
-    void initLocalDrivers();
-    void initRemoteDrivers();
-
-    void saveLocalDrivers();
-    void saveRemoteDrivers();
+    void loadDrivers();
+    void loadProfiles();
 
     void processLocalDevice(ISD::GDInterface*);
     void processRemoteDevice(ISD::GDInterface*);
@@ -244,14 +228,21 @@ protected slots:
 
     bool useGuideHead;
     bool useST4;
-    bool guideStarted;
-    bool ccdStarted;
-    bool scopeRegistered;
-    bool remoteCCDRegistered;
-    bool remoteGuideRegistered;
 
-    ISD::GDInterface *scope, *ccd, *guider, *focuser, *filter, *aux1, *aux2, *aux3, *aux4, *dome, *ao, *weather, *dustCap, *lightBox;
-    DriverInfo *scope_di, *ccd_di, *guider_di, *filter_di, *focuser_di, *aux1_di, *aux2_di, *aux3_di,*aux4_di, *ao_di, *dome_di, *weather_di, *remote_indi;
+    // Containers
+
+    // All Drivers
+    QHash<QString, DriverInfo *> driversList;
+
+    // All managed drivers
+    QList<DriverInfo *> managedDrivers;
+
+    // All generic devices
+    QList<ISD::GDInterface*> genericDevices;
+
+    // All Managed devices
+    QMap<DeviceFamily, ISD::GDInterface*> managedDevices;
+    QList<ISD::GDInterface*> findDevices(DeviceFamily type);
 
     Ekos::Capture *captureProcess;
     Ekos::Focus *focusProcess;
@@ -263,19 +254,46 @@ protected slots:
     Ekos::Weather *weatherProcess;
     Ekos::DustCap *dustCapProcess;
 
-    QString guiderCCDName;
-    QString primaryCCDName;
-    bool localMode, ccdDriverSelected;
+    bool localMode, isStarted, remoteManagerStart;
 
     int nDevices, nRemoteDevices;
-    QAtomicInt nConnectedDevices;
-    QList<DriverInfo *> managedDevices;
-    QHash<QString, DriverInfo *> driversList;
+    //QAtomicInt nConnectedDevices;
+
     QStringList logText;
     KPageWidgetItem *ekosOption;
     CommunicationStatus ekosStartingStatus, indiConnectionStatus;
 
+    QStandardItemModel *profileModel;
+    QList<ProfileInfo *> profiles;
+
+    ProfileInfo * getCurrentProfile();
+    void updateProfileLocation(ProfileInfo *pi);
+
+    // Mount Summary
+    QProgressIndicator *mountPI;
+
+    // Capture Summary
+    QTime imageCountDown;
+    QTime overallCountDown;
+    QTime sequenceCountDown;
+    QTimer countdownTimer;
+    QPixmap *previewPixmap;
+    QProgressIndicator *capturePI;
+
+    // Focus Summary
+    QProgressIndicator *focusPI;
+    QPixmap *focusStarPixmap;
+    //QPixmap *focusProfilePixmap;
+    //QTemporaryFile focusStarFile;
+    //QTemporaryFile focusProfileFile;
+
+    // Guide Summary
+    QProgressIndicator *guidePI;
+    QPixmap *guideStarPixmap;
+    //QPixmap *guideProfilePixmap;
+    //QTemporaryFile guideStarFile;
+    //QTemporaryFile guideProfileFile;
+
 };
 
-
 #endif // EKOS_H
