@@ -15,7 +15,6 @@
 #include <QHash>
 #include <QStringList>
 #include <QDialog>
-#include <unistd.h>
 
 #include <lilxml.h>
 #include "ui_drivermanager.h"
@@ -30,8 +29,6 @@ class ServerManager;
 class ClientManager;
 class DriverInfo;
 
-
-
 class DriverManagerUI : public QFrame, public Ui::DriverManager
 {
     Q_OBJECT
@@ -125,19 +122,15 @@ public:
 
     void clearServers();
 
-protected:
-    void closeEvent(QCloseEvent *);
-    void hideEvent(QHideEvent *);
-    void showEvent(QShowEvent *);
-
 private:
-    DriverManager();
+    DriverManager(QWidget *parent);
+    ~DriverManager();
 
     static DriverManager * _DriverManager;
 
     ServerMode connectionMode;
-    DriverManagerUI *ui;
 
+    DriverManagerUI *ui;
     QList<DriverInfo *> driversList;
     QList<ServerManager *> servers;
     QList<ClientManager *> clients;
