@@ -25,6 +25,11 @@
 #include "fitsviewer/fitsdata.h"
 #endif
 
+#ifdef HAVE_LIBRAW
+#include <libraw/libraw.h>
+#endif
+
+#include "driverinfo.h"
 #include "clientmanager.h"
 #include "streamwg.h"
 #include "indiccd.h"
@@ -37,15 +42,18 @@
 
 #include "Options.h"
 
+
 const int MAX_FILENAME_LEN = 1024;
+const QStringList RAWFormats = { "cr2", "crw", "nef", "raf", "dng" };
 
 namespace ISD
 {
 
-CCDChip::CCDChip(INDI::BaseDevice *bDevice, ClientManager *cManager, ChipType cType)
+CCDChip::CCDChip(ISD::CCD *ccd, ChipType cType)
 {
-    baseDevice    = bDevice;
-    clientManager = cManager;
+    baseDevice    = ccd->getBaseDevice();
+    clientManager = ccd->getDriverInfo()->getClientManager();
+    parentCCD     = ccd;
     type          = cType;
     batchMode     = false;
     displayFITS   = true;
@@ -57,12 +65,12 @@ CCDChip::CCDChip(INDI::BaseDevice *bDevice, ClientManager *cManager, ChipType cT
     captureMode   = FITS_NORMAL;
     captureFilter = FITS_NONE;
 
-    fx=fy=fw=fh=0;
+    //fx=fy=fw=fh=0;
 
     normalImage = focusImage = guideImage = calibrationImage = NULL;
 }
 
-FITSView * CCDChip::getImage(FITSMode imageType)
+FITSView * CCDChip::getImageView(FITSMode imageType)
 {
     switch (imageType)
     {
@@ -82,6 +90,10 @@ FITSView * CCDChip::getImage(FITSMode imageType)
         return calibrationImage;
         break;
 
+    case FITS_ALIGN:
+        return alignImage;
+        break;
+
     default:
         break;
     }
@@ -90,41 +102,38 @@ FITSView * CCDChip::getImage(FITSMode imageType)
 
 }
 
-void CCDChip::setImage(FITSView *image, FITSMode imageType)
+void CCDChip::setImageView(FITSView *image, FITSMode imageType)
 {
-
     switch (imageType)
     {
     case FITS_NORMAL:
         normalImage = image;
-        if (normalImage)
-            imageData = normalImage->getImageData();
-        if (KStars::Instance()->ekosManager()->alignModule() && KStars::Instance()->ekosManager()->alignModule()->fov())
-            KStars::Instance()->ekosManager()->alignModule()->fov()->setImage(normalImage->getDisplayImage()->copy());
         break;
 
     case FITS_FOCUS:
         focusImage = image;
-        if (focusImage)
-            imageData = focusImage->getImageData();
         break;
 
     case FITS_GUIDE:
         guideImage = image;
-        if (guideImage)
-            imageData = guideImage->getImageData();
         break;
 
     case FITS_CALIBRATE:
         calibrationImage = image;
-        if (calibrationImage)
-            imageData = calibrationImage->getImageData();
+        break;
+
+    case FITS_ALIGN:
+        alignImage = image;
+        if (KStars::Instance()->ekosManager()->alignModule() && KStars::Instance()->ekosManager()->alignModule()->fov())
+            KStars::Instance()->ekosManager()->alignModule()->fov()->setImage(alignImage->getDisplayImage()->copy());
         break;
 
     default:
         break;
     }
 
+    if (image)
+        imageData = image->getImageData();
 }
 
 bool CCDChip::getFrameMinMax(int *minX, int *maxX, int *minY, int *maxY, int *minW, int *maxW, int *minH, int *maxH)
@@ -150,31 +159,31 @@ bool CCDChip::getFrameMinMax(int *minX, int *maxX, int *minY, int *maxY, int *mi
     if (arg == NULL)
         return false;
 
-    *minX = arg->min;
-    *maxX = arg->max;
+    if (minX) *minX = arg->min;
+    if (maxX) *maxX = arg->max;
 
 
     arg = IUFindNumber(frameProp, "Y");
     if (arg == NULL)
         return false;
 
-    *minY = arg->min;
-    *maxY = arg->max;
+    if (minY) *minY = arg->min;
+    if (maxY) *maxY = arg->max;
 
     arg = IUFindNumber(frameProp, "WIDTH");
     if (arg == NULL)
         return false;
 
-    *minW = arg->min;
-    *maxW = arg->max;
+    if (minW) *minW = arg->min;
+    if (maxW) *maxW = arg->max;
 
 
     arg = IUFindNumber(frameProp, "HEIGHT");
     if (arg == NULL)
         return false;
 
-    *minH = arg->min;
-    *maxH = arg->max;
+    if (minH) *minH = arg->min;
+    if (maxH) *maxH = arg->max;
 
     return true;
 
@@ -395,7 +404,7 @@ bool CCDChip::canAbort() const
     return CanAbort;
 }
 
-bool CCDChip::getFocusFrame(int *x, int *y, int *w, int *h)
+/*bool CCDChip::getFocusFrame(int *x, int *y, int *w, int *h)
 {
     *x = fx;
     *y = fy;
@@ -413,7 +422,7 @@ bool CCDChip::setFocusFrame(int x, int y, int w, int h)
     fh=h;
 
     return true;
-}
+}*/
 
 void CCDChip::setCanAbort(bool value)
 {
@@ -469,7 +478,7 @@ QStringList CCDChip::getISOList() const
 bool CCDChip::isCapturing()
 {
     INumberVectorProperty *expProp = NULL;
-    
+
     switch (type)
     {
     case PRIMARY_CCD:
@@ -782,9 +791,9 @@ CCD::CCD(GDInterface *iPtr) : DeviceDecorator(iPtr)
     ST4Driver = NULL;
     nextSequenceID  = 0 ;
 
-    primaryChip = new CCDChip(baseDevice, clientManager, CCDChip::PRIMARY_CCD);
+    primaryChip = new CCDChip(this, CCDChip::PRIMARY_CCD);
 
-    normalTabID = calibrationTabID = focusTabID = guideTabID = -1;
+    normalTabID = calibrationTabID = focusTabID = guideTabID = alignTabID = -1;
     guideChip   = NULL;
 
 }
@@ -804,7 +813,7 @@ void CCD::registerProperty(INDI::Property *prop)
     if (!strcmp(prop->getName(), "GUIDER_EXPOSURE"))
     {
         HasGuideHead = true;
-        guideChip = new CCDChip(baseDevice, clientManager, CCDChip::GUIDE_CCD);
+        guideChip = new CCDChip(this, CCDChip::GUIDE_CCD);
     }
     else if (!strcmp(prop->getName(), "CCD_FRAME_TYPE"))
     {
@@ -998,11 +1007,10 @@ void CCD::processSwitch(ISwitchVectorProperty *svp)
 
         }
 
-        streamWindow->disconnect();
-        connect(streamWindow, SIGNAL(hidden()), this, SLOT(StreamWindowHidden()));
-
         if (streamWindow)
         {
+            connect(streamWindow, SIGNAL(hidden()), this, SLOT(StreamWindowHidden()), Qt::UniqueConnection);
+
             if (svp->sp[0].s == ISS_ON)
                 streamWindow->enableStream(true);
             else
@@ -1051,9 +1059,6 @@ void CCD::processText(ITextVectorProperty *tvp)
 
 void CCD::processBLOB(IBLOB* bp)
 {
-
-    enum blobType { BLOB_IMAGE, BLOB_FITS, BLOB_CR2, BLOB_OTHER} BType;
-
     BType = BLOB_OTHER;
 
     QString format(bp->format);
@@ -1087,8 +1092,8 @@ void CCD::processBLOB(IBLOB* bp)
         BType = BLOB_IMAGE;
     else if (format.contains("fits"))
         BType = BLOB_FITS;
-    else if (format.contains("cr2"))
-        BType = BLOB_CR2;
+    else if (RAWFormats.contains(fmt))
+        BType = BLOB_RAW;
 
     if (BType == BLOB_OTHER)
     {
@@ -1113,13 +1118,15 @@ void CCD::processBLOB(IBLOB* bp)
     int nr, n=0;
     QTemporaryFile tmpFile(QDir::tempPath() + "/fitsXXXXXX");
 
-    if (currentDir.endsWith('/'))
-        currentDir.truncate(sizeof(currentDir)-1);
+    //if (currentDir.endsWith('/'))
+    //currentDir.truncate(currentDir.size()-1);
 
     if (QDir(currentDir).exists() == false)
         QDir().mkpath(currentDir);
 
-    QString filename(currentDir + '/');
+    QString filename(currentDir);
+    if (filename.endsWith('/') == false)
+        filename.append('/');
 
     // Create temporary name if ANY of the following conditions are met:
     // 1. file is preview or batch mode is not enabled
@@ -1155,7 +1162,7 @@ void CCD::processBLOB(IBLOB* bp)
         if (ISOMode == false)
             filename += seqPrefix + (seqPrefix.isEmpty() ? "" : "_") +  QString("%1.%2").arg(QString().sprintf("%03d", nextSequenceID)).arg(QString(fmt));
         else
-            filename += seqPrefix + (seqPrefix.isEmpty() ? "" : "_") + QString("%1_%2.%3").arg(QString().sprintf("%03d", nextSequenceID)).arg(ts).arg(QString(fmt));
+            filename += seqPrefix + (seqPrefix.isEmpty() ? "" : "_") + QString("%1_%2.%3").arg(ts).arg(QString().sprintf("%03d", nextSequenceID)).arg(QString(fmt));
 
         QFile fits_temp_file(filename);
         if (!fits_temp_file.open(QIODevice::WriteOnly))
@@ -1178,12 +1185,14 @@ void CCD::processBLOB(IBLOB* bp)
 
     // store file name
     strncpy(BLOBFilename, filename.toLatin1(), MAXINDIFILENAME);
+    bp->aux1 = &BType;
     bp->aux2 = BLOBFilename;
 
     if (targetChip->getCaptureMode() == FITS_NORMAL && targetChip->isBatchMode() == true)
         KStars::Instance()->statusBar()->showMessage( i18n("%1 file saved to %2", QString(fmt).toUpper(), filename ), 0);
 
-    KNotification::event( QLatin1String( "FITSReceived" ) , i18n("FITS file is received"));
+    // FIXME: Why is this leaking memory in Valgrind??!
+    KNotification::event( QLatin1String( "FITSReceived" ) , i18n("Image file is received"));
 
     /*if (targetChip->showFITS() == false && targetChip->getCaptureMode() == FITS_NORMAL)
     {
@@ -1191,55 +1200,102 @@ void CCD::processBLOB(IBLOB* bp)
         return;
     }*/
 
-    if (BType == BLOB_IMAGE || BType == BLOB_CR2)
+    if (BType == BLOB_IMAGE || BType == BLOB_RAW)
     {
-        if (BType == BLOB_CR2)
-        {
-            if (QStandardPaths::findExecutable("dcraw").isEmpty() == false && QStandardPaths::findExecutable("cjpeg").isEmpty() == false)
-            {
-                QProcess dcraw;
-                QString rawFileName = filename;
-                rawFileName = rawFileName.remove(0, rawFileName.lastIndexOf(QLatin1Literal("/")));
-                QString templateName = QString("%1/%2.XXXXXX").arg(QDir::tempPath()).arg(rawFileName);
-                QTemporaryFile jpgPreview(templateName);
-                jpgPreview.setAutoRemove(false);
-                jpgPreview.open();
-                jpgPreview.close();
-                QString jpeg_filename = jpgPreview.fileName();
+        if (BType == BLOB_RAW)
+        {           
+            #ifdef HAVE_LIBRAW
 
-                QString cmd = QString("/bin/sh -c \"dcraw -c -q 0 -w -H 5 -b 8 %1 | cjpeg -quality 80 > %2\"").arg(filename).arg(jpeg_filename);
-                dcraw.start(cmd);
-                dcraw.waitForFinished();
-                filename = jpeg_filename;
-            }
-            else
+            QString rawFileName = filename;
+            rawFileName = rawFileName.remove(0, rawFileName.lastIndexOf(QLatin1Literal("/")));
+            QString templateName = QString("%1/%2.XXXXXX").arg(QDir::tempPath()).arg(rawFileName);
+            QTemporaryFile imgPreview(templateName);
+            imgPreview.setAutoRemove(false);
+            imgPreview.open();
+            imgPreview.close();
+            QString preview_filename = imgPreview.fileName();
+
+            int  ret=0;
+            // Creation of image processing object
+            LibRaw RawProcessor;
+
+            // Let us open the file
+            if( (ret = RawProcessor.open_file(filename.toLatin1().data())) != LIBRAW_SUCCESS)
             {
-                KStars::Instance()->statusBar()->showMessage(i18n("Unable to find dcraw and cjpeg. Please install the required tools to convert CR2 to JPEG."));
+                KStars::Instance()->statusBar()->showMessage(i18n("Cannot open %1: %2", rawFileName, libraw_strerror(ret)));
+                RawProcessor.recycle();
                 emit BLOBUpdated(bp);
                 return;
             }
+
+            // Let us unpack the image
+            /*if( (ret = RawProcessor.unpack() ) != LIBRAW_SUCCESS)
+            {
+                KStars::Instance()->statusBar()->showMessage(i18n("Cannot unpack_thumb %1: %2", rawFileName, libraw_strerror(ret)));
+                if(LIBRAW_FATAL_ERROR(ret))
+                {
+                    RawProcessor.recycle();
+                    emit BLOBUpdated(bp);
+                    return;
+                }
+                // if there has been a non-fatal error, we will try to continue
+            }*/
+
+            // Let us unpack the thumbnail
+            if( (ret = RawProcessor.unpack_thumb() ) != LIBRAW_SUCCESS)
+            {
+                KStars::Instance()->statusBar()->showMessage(i18n("Cannot unpack_thumb %1: %2", rawFileName, libraw_strerror(ret)));
+                    RawProcessor.recycle();
+                    emit BLOBUpdated(bp);
+                    return;
+            }
+            else
+                // We have successfully unpacked the thumbnail, now let us write it to a file
+            {
+                //snprintf(thumbfn,sizeof(thumbfn),"%s.%s",av[i],T.tformat == LIBRAW_THUMBNAIL_JPEG ? "thumb.jpg" : "thumb.ppm");
+                if( LIBRAW_SUCCESS != (ret = RawProcessor.dcraw_thumb_writer(preview_filename.toLatin1().data())))
+                {
+                    KStars::Instance()->statusBar()->showMessage(i18n("Cannot write %s %1: %2", preview_filename, libraw_strerror(ret)));
+                    RawProcessor.recycle();
+                    emit BLOBUpdated(bp);
+                    return;
+                }
+            }
+
+            filename = preview_filename;
+
+            #else
+                // Silenty fail if KStars was not compiled with libraw
+                //KStars::Instance()->statusBar()->showMessage(i18n("Unable to find dcraw and cjpeg. Please install the required tools to convert CR2/NEF to JPEG."));
+                emit BLOBUpdated(bp);
+                return;
+            #endif
         }
 
+        // store file name in
+        strncpy(BLOBFilename, filename.toLatin1(), MAXINDIFILENAME);
+        bp->aux1 = &BType;
+        bp->aux2 = BLOBFilename;
+
         if (imageViewer.isNull())
             imageViewer = new ImageViewer(getDeviceName(), KStars::Instance());
 
         imageViewer->loadImage(filename);
-        QFile::remove(filename);
     }
     // Unless we have cfitsio, we're done.
 #ifdef HAVE_CFITSIO
-    if (BType == BLOB_FITS && targetChip->getCaptureMode() != FITS_WCSM)
+    if (BType == BLOB_FITS)
     {
-        QUrl fileURL(filename);
+        QUrl fileURL = QUrl::fromLocalFile(filename);
 
         if (fv.isNull())
         {
-            normalTabID = calibrationTabID = focusTabID = guideTabID = -1;
+            normalTabID = calibrationTabID = focusTabID = guideTabID = alignTabID = -1;
 
             if (Options::singleWindowCapturedFITS())
                 fv = KStars::Instance()->genericFITSViewer();
             else
-                fv = new FITSViewer(KStars::Instance());
+                fv = new FITSViewer(Options::independentWindowFITS() ? NULL : KStars::Instance());
 
             //connect(fv, SIGNAL(destroyed()), this, SLOT(FITSViewerDestroyed()));
             //connect(fv, SIGNAL(destroyed()), this, SIGNAL(FITSViewerClosed()));
@@ -1258,23 +1314,18 @@ void CCD::processBLOB(IBLOB* bp)
                 previewTitle = i18n("Preview");
         }
 
-
         int tabRC = -1;
 
         switch (targetChip->getCaptureMode())
         {
         case FITS_NORMAL:
         {
-            FITSScale actualFilter = captureFilter;
-            if (Options::autoStretch() && captureFilter == FITS_NONE)
-                actualFilter = FITS_AUTO_STRETCH;
-
             if (normalTabID == -1 || Options::singlePreviewFITS() == false)
-                tabRC = fv->addFITS(&fileURL, FITS_NORMAL, actualFilter, previewTitle);
-            else if (fv->updateFITS(&fileURL, normalTabID, actualFilter) == false)
+                tabRC = fv->addFITS(&fileURL, FITS_NORMAL, captureFilter, previewTitle);
+            else if (fv->updateFITS(&fileURL, normalTabID, captureFilter) == false)
             {
                 fv->removeFITS(normalTabID);
-                tabRC = fv->addFITS(&fileURL, FITS_NORMAL, actualFilter, previewTitle);
+                tabRC = fv->addFITS(&fileURL, FITS_NORMAL, captureFilter, previewTitle);
             }
             else
                 tabRC = normalTabID;
@@ -1282,17 +1333,21 @@ void CCD::processBLOB(IBLOB* bp)
             if (tabRC >= 0)
             {
                 normalTabID = tabRC;
-                targetChip->setImage(fv->getView(normalTabID), FITS_NORMAL);
+                targetChip->setImageView(fv->getView(normalTabID), FITS_NORMAL);
 
-                emit newImage(fv->getView(normalTabID)->getDisplayImage());
+                emit newImage(fv->getView(normalTabID)->getDisplayImage(), targetChip);
             }
             else
+            {
                 // If opening file fails, we treat it the same as exposure failure and recapture again if possible
                 emit newExposureValue(targetChip, 0, IPS_ALERT);
+                return;
+            }
         }
             break;
 
         case FITS_FOCUS:
+            /*
             if (focusTabID == -1)
                 tabRC = fv->addFITS(&fileURL, FITS_FOCUS, captureFilter);
             else if (fv->updateFITS(&fileURL, focusTabID, captureFilter) == false)
@@ -1306,13 +1361,41 @@ void CCD::processBLOB(IBLOB* bp)
             if (tabRC >= 0)
             {
                 focusTabID = tabRC;
-                targetChip->setImage(fv->getView(focusTabID), FITS_FOCUS);
+                targetChip->setImageView(fv->getView(focusTabID), FITS_FOCUS);
+
+                emit newImage(fv->getView(focusTabID)->getDisplayImage(), targetChip);
             }
             else
+            {
                 emit newExposureValue(targetChip, 0, IPS_ALERT);
+                // If there is problem loading image then BLOB is not valid so let's return
+                return;
+            }
+            */
+
+        {
+            FITSView *focusView = targetChip->getImageView(FITS_FOCUS);
+            if (focusView)
+            {
+                focusView->setFilter(captureFilter);
+                bool imageLoad = focusView->loadFITS(filename, true);
+                if (imageLoad)
+                {
+                    //focusView->rescale(ZOOM_FIT_WINDOW);
+                    focusView->updateFrame();
+                    emit newImage(focusView->getDisplayImage(), targetChip);
+                }
+                else
+                {
+                    emit newExposureValue(targetChip, 0, IPS_ALERT);
+                    return;
+                }
+            }
+        }
             break;
 
         case FITS_GUIDE:
+            /*
             if (guideTabID == -1)
                 tabRC = fv->addFITS(&fileURL, FITS_GUIDE, captureFilter);
             else if (fv->updateFITS(&fileURL, guideTabID, captureFilter) == false)
@@ -1326,10 +1409,34 @@ void CCD::processBLOB(IBLOB* bp)
             if (tabRC >= 0)
             {
                 guideTabID = tabRC;
-                targetChip->setImage(fv->getView(guideTabID), FITS_GUIDE);
+                targetChip->setImageView(fv->getView(guideTabID), FITS_GUIDE);
+
+                emit newImage(fv->getView(guideTabID)->getDisplayImage(), targetChip);
             }
             else
+            {
                 emit newExposureValue(targetChip, 0, IPS_ALERT);
+                return;
+            }*/
+        {
+            FITSView *guideView = targetChip->getImageView(FITS_GUIDE);
+            if (guideView)
+            {
+                guideView->setFilter(captureFilter);
+                bool imageLoad = guideView->loadFITS(filename, true);
+                if (imageLoad)
+                {
+                    //guideView->rescale(ZOOM_FIT_WINDOW);
+                    guideView->updateFrame();
+                    emit newImage(guideView->getDisplayImage(), targetChip);
+                }
+                else
+                {
+                    emit newExposureValue(targetChip, 0, IPS_ALERT);
+                    return;
+                }
+            }
+        }
             break;
 
         case FITS_CALIBRATE:
@@ -1346,10 +1453,36 @@ void CCD::processBLOB(IBLOB* bp)
             if (tabRC >= 0)
             {
                 calibrationTabID = tabRC;
-                targetChip->setImage(fv->getView(calibrationTabID), FITS_CALIBRATE);
+                targetChip->setImageView(fv->getView(calibrationTabID), FITS_CALIBRATE);
             }
             else
+            {
                 emit newExposureValue(targetChip, 0, IPS_ALERT);
+                return;
+            }
+            break;
+
+        case FITS_ALIGN:
+            if (alignTabID == -1)
+                tabRC = fv->addFITS(&fileURL, FITS_ALIGN, captureFilter);
+            else if (fv->updateFITS(&fileURL, alignTabID, captureFilter) == false)
+            {
+                fv->removeFITS(alignTabID);
+                tabRC = fv->addFITS(&fileURL, FITS_ALIGN, captureFilter);
+            }
+            else
+                tabRC = alignTabID;
+
+            if (tabRC >= 0)
+            {
+                alignTabID = tabRC;
+                targetChip->setImageView(fv->getView(alignTabID), FITS_ALIGN);
+            }
+            else
+            {
+                emit newExposureValue(targetChip, 0, IPS_ALERT);
+                return;
+            }
             break;
 
 
@@ -1358,8 +1491,8 @@ void CCD::processBLOB(IBLOB* bp)
 
         }
 
-        fv->show();
-
+        if (targetChip->getCaptureMode() != FITS_GUIDE && targetChip->getCaptureMode() != FITS_FOCUS)
+            fv->show();
     }
 #endif
 
@@ -1401,7 +1534,7 @@ void CCD::addFITSKeywords(QString filename)
 void CCD::FITSViewerDestroyed()
 {
     fv = NULL;
-    normalTabID = calibrationTabID = focusTabID = guideTabID = -1;
+    normalTabID = calibrationTabID = focusTabID = guideTabID = alignTabID = -1;
 }
 
 void CCD::StreamWindowHidden()
@@ -1539,7 +1672,7 @@ bool CCD::configureRapidGuide(CCDChip *targetChip, bool autoLoop, bool sendImage
     return true;
 }
 
-void CCD::updateUploadSettings()
+void CCD::updateUploadSettings(const QString &remoteDir)
 {
     QString filename = seqPrefix + (seqPrefix.isEmpty() ? "" : "_") +  QString("XXX");
 
@@ -1550,15 +1683,13 @@ void CCD::updateUploadSettings()
     if (uploadSettingsTP)
     {
         uploadT = IUFindText(uploadSettingsTP, "UPLOAD_DIR");
-        if (uploadT)
-            IUSaveText(uploadT, fitsDir.toLatin1().constData());
+        if (uploadT && remoteDir.isEmpty() == false)
+            IUSaveText(uploadT, remoteDir.toLatin1().constData());
 
         uploadT = IUFindText(uploadSettingsTP, "UPLOAD_PREFIX");
         if (uploadT)
             IUSaveText(uploadT, filename.toLatin1().constData());
 
-
-
         clientManager->sendNewText(uploadSettingsTP);
     }
 }
