@@ -22,7 +22,7 @@
 
 #include "ksnumbers.h"
 #include "ksuserdb.h"
-#include "datahandlers/catalogdb.h"
+#include "catalogdb.h"
 
 #include <QList>
 #include <QMap>
@@ -32,9 +32,10 @@
 #include "colorscheme.h"
 #include "kstarsdatetime.h"
 #include "simclock.h"
+#ifndef KSTARS_LITE
 #include "oal/oal.h"
 #include "oal/log.h"
-
+#endif
 #define MINZOOM 250.
 #define MAXZOOM 5000000.
 #define DEFAULTZOOM 2000.
@@ -52,7 +53,18 @@ class FOV;
 class ObservingList;
 
 class TimeZoneRule;
-struct ADVTreeData;
+
+#ifdef KSTARS_LITE
+//Will go away when details window will be implemented in KStars Lite
+struct ADVTreeData
+{
+    QString Name;
+    QString Link;
+    int Type;
+};
+#else
+class ADVTreeData;
+#endif
 
 
 /** @class KStarsData
@@ -80,6 +92,7 @@ public:
     friend class SkyMap;
     // FIXME: uses geoList and changes it.
     friend class LocationDialog;
+    friend class LocationDialogLite;
 
     static KStarsData* Create( );
 
@@ -142,6 +155,9 @@ public:
     /** @return pointer to the ColorScheme object */
     ColorScheme *colorScheme() { return &CScheme; }
 
+    /** @return name of current color scheme **/
+    Q_INVOKABLE QString colorSchemeName() { return CScheme.fileName(); }
+
     /** @return pointer to the KSUserDB object */
     KSUserDB *userdb() { return &m_ksuserdb; }
 
@@ -149,10 +165,10 @@ public:
     CatalogDB *catalogdb() { return &m_catalogdb; }
 
     /** @return pointer to the simulation Clock object */
-    SimClock *clock() { return &Clock; }
+    Q_INVOKABLE SimClock *clock() { return &Clock; }
 
     /** @return pointer to the local sidereal time: a dms object */
-    dms *lst() { return &LST; }
+    CachingDms *lst() { return &LST; }
 
     /** @return pointer to the GeoLocation object*/
     GeoLocation *geo() { return &m_Geo; }
@@ -208,7 +224,7 @@ public:
       *@return the list of available FOVs
       */
     inline const QList<FOV*> getAvailableFOVs() const { return availFOVs; }
-
+    #ifndef KSTARS_LITE
     /** Return log object */
     OAL::Log *logObject() { return m_LogObject; }
 
@@ -220,7 +236,7 @@ public:
     ImageExporter * imageExporter();
 
     Execute* executeSession();
-
+    #endif
     /*@short Increments the updateID, forcing a recomputation of star positions as well */
     unsigned int incUpdateID();
 
@@ -231,7 +247,7 @@ public:
 
 signals:
     /** Signal that specifies the text that should be drawn in the KStarsSplash window. */
-    void progressText( const QString& );
+    void progressText( const QString& text );
 
     /** Should be used to refresh skymap. */
     void skyUpdate( bool );
@@ -339,10 +355,12 @@ private:
     KSUserDB m_ksuserdb;
     CatalogDB m_catalogdb;
     ColorScheme CScheme;
-    OAL::Log *m_LogObject;
+    #ifndef KSTARS_LITE
     ObservingList *m_ObservingList;
+    OAL::Log *m_LogObject;
     Execute *m_Execute;
     ImageExporter *m_ImageExporter;
+    #endif
 
     //EquipmentWriter *m_equipmentWriter;
 
@@ -352,7 +370,7 @@ private:
 
     //KLocale *locale;
 
-    dms LST;
+    CachingDms LST;
 
     QKeySequence resumeKey;
 
