@@ -22,8 +22,11 @@
 #include <QProgressDialog>
 #include <QNetworkAccessManager>
 #include <QNetworkReply>
+#include <QtConcurrent>
 
+#ifndef KSTARS_LITE
 #include <KJobUiDelegate>
+#endif
 #include <KLocalizedString>
 
 #include "satellitegroup.h"
@@ -34,15 +37,24 @@
 
 SatellitesComponent::SatellitesComponent( SkyComposite *parent ) :
     SkyComponent( parent )
+{
+    QtConcurrent::run(this, &SatellitesComponent::loadData);
+}
+
+SatellitesComponent::~SatellitesComponent()
+{
+
+}
+
+void SatellitesComponent::loadData()
 {
     KSFileReader fileReader;
     QString line;
     QStringList group_infos;
-    
+
     if ( ! fileReader.open( "satellites.dat" ) ) return;
 
     emitProgressText( i18n("Loading satellites" ) );
-    
 
     while ( fileReader.hasMoreLines() ) {
         line = fileReader.readLine();
@@ -53,6 +65,7 @@ SatellitesComponent::SatellitesComponent( SkyComposite *parent ) :
     }
 
     objectNames(SkyObject::SATELLITE).clear();
+    objectLists(SkyObject::SATELLITE).clear();
 
     foreach( SatelliteGroup *group, m_groups )
     {
@@ -62,17 +75,13 @@ SatellitesComponent::SatellitesComponent( SkyComposite *parent ) :
             if ( sat->selected() && nameHash.contains(sat->name().toLower()) == false)
             {
                 objectNames(SkyObject::SATELLITE).append(sat->name());
+                objectLists(SkyObject::SATELLITE).append(QPair<QString, const SkyObject*>(sat->name(), sat));
                 nameHash[sat->name().toLower()] = sat;
             }
         }
     }
 }
 
-SatellitesComponent::~SatellitesComponent()
-{
-
-}
-
 bool SatellitesComponent::selected() {
     return Options::showSatellites();
 }
@@ -148,7 +157,7 @@ void SatellitesComponent::updateTLEs()
 
         if (response->error() == QNetworkReply::NoError)
         {
-            QFile file(group->tleFilename().path());
+            QFile file(group->tleFilename().toLocalFile());
             if (file.open(QFile::WriteOnly))
             {
                 file.write(response->readAll());
@@ -159,14 +168,22 @@ void SatellitesComponent::updateTLEs()
             }
             else
             {
+            #ifndef KSTARS_LITE
                 KMessageBox::error(0, file.errorString());
+            #else
+                qDebug() << file.errorString();
+            #endif
                 return;
             }
 
         }
         else
         {
-            KMessageBox::error(0, response->errorString());
+            #ifndef KSTARS_LITE
+                KMessageBox::error(0, response->errorString());
+            #else
+                qDebug() << response->errorString();
+            #endif
             return;
         }
     }
