@@ -47,6 +47,8 @@ public:
     KSPlanet* earth() { return m_Earth; }
     const QList<SkyObject*>& asteroids() const;
     const QList<SkyObject*>& comets() const;
+    const QList<SkyObject*>& planetObjects() const;
+    const QList<SkyObject*>& moons() const;
 
     bool selected();
 
@@ -62,6 +64,9 @@ public:
 
     AsteroidsComponent* asteroidsComponent();
 
+    QList<PlanetMoonsComponent *> planetMoonsComponent() const;
+
+    const QList<SolarSystemSingleComponent *>& planets() const;
 private:
     KSPlanet *m_Earth;
     KSSun *m_Sun;
@@ -69,6 +74,9 @@ private:
     PlanetMoonsComponent *m_JupiterMoons;
     AsteroidsComponent *m_AsteroidsComponent;
     CometsComponent *m_CometsComponent;
+    QList<SolarSystemSingleComponent *> m_planets;
+    QList<SkyObject *> m_planetObjects;
+    QList<SkyObject *> m_moons;
 };
 
 #endif
